@@ -29,23 +29,9 @@
  */
 static GbWidget gbwidget;
 
-static gchar *Policy = "GtkColorSelection::policy";
+static gchar *HasOpacity = "GtkColorSelection::has_opacity_control";
+static gchar *HasPalette = "GtkColorSelection::has_palette";
 
-static const gchar *GbPolicyChoices[] =
-{"Continuous", "Discontinuous", "Delayed",
- NULL};
-static const gint GbPolicyValues[] =
-{
-  GTK_UPDATE_CONTINUOUS,
-  GTK_UPDATE_DISCONTINUOUS,
-  GTK_UPDATE_DELAYED
-};
-static const gchar *GbPolicySymbols[] =
-{
-  "GTK_UPDATE_CONTINUOUS",
-  "GTK_UPDATE_DISCONTINUOUS",
-  "GTK_UPDATE_DELAYED"
-};
 
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
@@ -59,13 +45,11 @@ static const gchar *GbPolicySymbols[] =
  * function, and call data->callback with your new widget when it is done.
  * If the widget needs a special destroy handler, add a signal here.
  */
-/*
-   GtkWidget*
-   gb_color_selection_new(GbWidgetNewData *data)
-   {
-
-   }
- */
+GtkWidget*
+gb_color_selection_new(GbWidgetNewData *data)
+{
+  return gtk_color_selection_new ();
+}
 
 
 /*
@@ -75,13 +59,13 @@ static void
 gb_color_selection_create_properties (GtkWidget * widget, GbWidgetCreateArgData
 				      * data)
 {
-  property_add_choice (Policy, _("Update Policy:"),
-		       _("The update policy of the color selection scales"),
-		       GbPolicyChoices);
+  property_add_bool (HasOpacity, _("Opacity Control:"),
+		     _("If the opacity control is shown"));
+  property_add_bool (HasPalette, _("Palette:"),
+		     _("If the palette is shown"));
 }
 
 
-
 /*
  * Gets the properties of the widget. This is used for both displaying the
  * properties in the property editor, and also for saving the properties.
@@ -89,17 +73,13 @@ gb_color_selection_create_properties (GtkWidget * widget, GbWidgetCreateArgData
 static void
 gb_color_selection_get_properties (GtkWidget * widget, GbWidgetGetArgData * data)
 {
-  gint i;
-
-  for (i = 0; i < sizeof (GbPolicyValues) / sizeof (GbPolicyValues[0]); i++)
-    {
-      if (GbPolicyValues[i] == GTK_COLOR_SELECTION (widget)->policy)
-	gb_widget_output_choice (data, Policy, i, GbPolicySymbols[i]);
-    }
+  gb_widget_output_bool (data, HasOpacity,
+			 gtk_color_selection_get_has_opacity_control (GTK_COLOR_SELECTION (widget)));
+  gb_widget_output_bool (data, HasPalette,
+			 gtk_color_selection_get_has_palette (GTK_COLOR_SELECTION (widget)));
 }
 
 
-
 /*
  * Sets the properties of the widget. This is used for both applying the
  * properties changed in the property editor, and also for loading.
@@ -107,28 +87,23 @@ gb_color_selection_get_properties (GtkWidget * widget, GbWidgetGetArgData * data
 static void
 gb_color_selection_set_properties (GtkWidget * widget, GbWidgetSetArgData * data)
 {
-  gint i;
-  gchar *policy;
+  gboolean has_opacity, has_palette;
 
-  policy = gb_widget_input_choice (data, Policy);
+  has_opacity = gb_widget_input_bool (data, HasOpacity);
   if (data->apply)
     {
-      for (i = 0; i < sizeof (GbPolicyValues) / sizeof (GbPolicyValues[0]); i
-	   ++)
-	{
-	  if (!strcmp (policy, GbPolicyChoices[i])
-	      || !strcmp (policy, GbPolicySymbols[i]))
-	    {
-	      gtk_color_selection_set_update_policy (GTK_COLOR_SELECTION (widget),
-						     GbPolicyValues[i]);
-	      break;
-	    }
-	}
+      gtk_color_selection_set_has_opacity_control (GTK_COLOR_SELECTION (widget), has_opacity);
+    }
+
+  has_palette = gb_widget_input_bool (data, HasPalette);
+  if (data->apply)
+    {
+      gtk_color_selection_set_has_palette (GTK_COLOR_SELECTION (widget),
+					   has_palette);
     }
 }
 
 
-
 /*
  * Adds menu items to a context menu which is just about to appear!
  * Add commands to aid in editing a GtkColorSelection, with signals pointing to
@@ -150,10 +125,10 @@ gb_color_selection_set_properties (GtkWidget * widget, GbWidgetSetArgData * data
  * there are some convenience functions to help.
  */
 static void
-gb_color_selection_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
-				 data)
+gb_color_selection_write_source (GtkWidget * widget,
+				 GbWidgetWriteSourceData * data)
 {
-  gint i;
+  gboolean has_opacity_control;
 
   if (data->create_widget)
     {
@@ -162,15 +137,17 @@ gb_color_selection_write_source (GtkWidget * widget, GbWidgetWriteSourceData *
 
   gb_widget_write_standard_source (widget, data);
 
-  if (GTK_COLOR_SELECTION (widget)->policy != GTK_UPDATE_CONTINUOUS)
+  /* Although the default is TRUE for the GtkColorSelection, it is FALSE in
+     the GtkColorSelectionDialog. So we always set it in the source code, since
+     this is used for both. */
+  has_opacity_control = gtk_color_selection_get_has_opacity_control (GTK_COLOR_SELECTION (widget));
+  source_add (data, "  gtk_color_selection_set_has_opacity_control (GTK_COLOR_SELECTION (%s), %s);\n",
+	      data->wname, has_opacity_control ? "TRUE" : "FALSE");
+
+  if (gtk_color_selection_get_has_palette (GTK_COLOR_SELECTION (widget)))
     {
-      for (i = 0; i < sizeof (GbPolicyValues) / sizeof (GbPolicyValues[0]); i
-	   ++)
-	{
-	  if (GbPolicyValues[i] == GTK_COLOR_SELECTION (widget)->policy)
-	    source_add (data, "  gtk_color_selection_set_update_policy (GTK_COLOR_SELECTION (%s), %s);\n",
-			data->wname, GbPolicySymbols[i]);
-	}
+      source_add (data, "  gtk_color_selection_set_has_palette (GTK_COLOR_SELECTION (%s), TRUE);\n",
+		  data->wname);
     }
 }
 
@@ -185,7 +162,8 @@ GbWidget *
 gb_color_selection_init ()
 {
   /* Initialise the GTK type */
-  gtk_color_selection_get_type ();
+  volatile GtkType type;
+  type = gtk_color_selection_get_type ();
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct (&gbwidget);
@@ -195,14 +173,14 @@ gb_color_selection_init ()
   gbwidget.tooltip = _("Color Selection");
 
   /* Fill in any functions that this GbWidget has */
+  gbwidget.gb_widget_new               = gb_color_selection_new;
   gbwidget.gb_widget_create_properties = gb_color_selection_create_properties;
   gbwidget.gb_widget_get_properties = gb_color_selection_get_properties;
   gbwidget.gb_widget_set_properties = gb_color_selection_set_properties;
   gbwidget.gb_widget_write_source = gb_color_selection_write_source;
-/*
-   gbwidget.gb_widget_new               = gb_color_selection_new;
-   gbwidget.gb_widget_create_popup_menu = gb_color_selection_create_popup_menu;
- */
+  /*
+    gbwidget.gb_widget_create_popup_menu = gb_color_selection_create_popup_menu;
+  */
 
   return &gbwidget;
 }
