@@ -29,23 +29,29 @@
  */
 static GbWidget gbwidget;
 
-/* These are the special children of the widget. */
-static gchar *FileSelOKButton = "FileSel:ok_button";
-static gchar *FileSelCancelButton = "FileSel:cancel_button";
-
 static gchar *Title = "FileSel|GtkWindow::title";
 static gchar *Type = "FileSel|GtkWindow::type";
-static gchar *Position = "FileSel|GtkWindow::position";
+static gchar *Position = "FileSel|GtkWindow::window_position";
 static gchar *Modal = "FileSel|GtkWindow::modal";
 static gchar *DefaultWidth = "FileSel|GtkWindow::default_width";
 static gchar *DefaultHeight = "FileSel|GtkWindow::default_height";
-static gchar *FileOps = "GtkFileSelection::show_file_op_buttons";
+static gchar *FileOps = "GtkFileSelection::show_fileops";
 static gchar *Shrink = "FileSel|GtkWindow::allow_shrink";
 static gchar *Grow = "FileSel|GtkWindow::allow_grow";
 static gchar *AutoShrink = "FileSel|GtkWindow::auto_shrink";
 static gchar *WMName = "FileSel|GtkWindow::wmclass_name";
 static gchar *WMClass = "FileSel|GtkWindow::wmclass_class";
 
+static gchar *Resizable = "FileSel|GtkWindow::resizable";
+static gchar *DestroyWithParent = "FileSel|GtkWindow::destroy_with_parent";
+static gchar *Icon = "FileSel|GtkWindow::icon";
+
+static gchar *Role = "FileSel|GtkWindow::role";
+static gchar *TypeHint = "FileSel|GtkWindow::type_hint";
+static gchar *SkipTaskbar = "FileSel|GtkWindow::skip_taskbar_hint";
+static gchar *SkipPager = "FileSel|GtkWindow::skip_pager_hint";
+static gchar *Decorated = "FileSel|GtkWindow::decorated";
+static gchar *Gravity = "FileSel|GtkWindow::gravity";
 
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
@@ -66,16 +72,23 @@ gb_file_selection_new (GbWidgetNewData * data)
 
   GtkFileSelection *filesel = GTK_FILE_SELECTION (new_widget);
 
+  /* We want it to be treated as a normal window. */
+  gtk_window_set_type_hint (GTK_WINDOW (new_widget),
+			    GDK_WINDOW_TYPE_HINT_NORMAL);
+
   gtk_signal_connect (GTK_OBJECT (new_widget), "delete_event",
 		      GTK_SIGNAL_FUNC (editor_close_window), NULL);
 
   gb_widget_create_from (filesel->ok_button,
 			 data->action == GB_CREATING ? "ok_button" : NULL);
-  gb_widget_set_child_name (filesel->ok_button, FileSelOKButton);
+  gb_widget_set_child_name (filesel->ok_button, GladeChildOKButton);
 
   gb_widget_create_from (filesel->cancel_button,
 			 data->action == GB_CREATING ? "cancel_button" : NULL);
-  gb_widget_set_child_name (filesel->cancel_button, FileSelCancelButton);
+  gb_widget_set_child_name (filesel->cancel_button, GladeChildCancelButton);
+
+  gtk_object_set_data (GTK_OBJECT (new_widget), TypeHint,
+		       GINT_TO_POINTER (GLADE_TYPE_HINT_DIALOG_INDEX));
 
   return new_widget;
 }
@@ -93,7 +106,10 @@ gb_file_selection_create_properties (GtkWidget * widget,
 					Title, Type, Position, Modal,
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, AutoShrink,
-					WMName, WMClass);
+					WMName, WMClass,
+					Resizable, DestroyWithParent, Icon,
+					Role, TypeHint, SkipTaskbar,
+					SkipPager, Decorated, Gravity);
   property_add_bool (FileOps, _("File Ops.:"),
 		     _("If the file operation buttons are shown"));
 }
@@ -112,7 +128,10 @@ gb_file_selection_get_properties (GtkWidget * widget,
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
   gb_widget_output_bool (data, FileOps,
 			 GTK_FILE_SELECTION (widget)->fileop_c_dir != NULL);
 }
@@ -133,7 +152,10 @@ gb_file_selection_set_properties (GtkWidget * widget,
 				     Title, Type, Position, Modal,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
-				     WMName, WMClass);
+				     WMName, WMClass,
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 
   show_fileops = gb_widget_input_bool (data, FileOps);
   if (data->apply)
@@ -188,7 +210,10 @@ gb_file_selection_write_source (GtkWidget * widget,
 				   NULL, Type, Position, Modal,
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
-				   WMName, WMClass);
+				   WMName, WMClass,
+				   Resizable, DestroyWithParent, Icon,
+				   Role, TypeHint, SkipTaskbar,
+				   SkipPager, Decorated, Gravity);
 
   if (GTK_FILE_SELECTION (widget)->fileop_c_dir == NULL)
     {
@@ -204,7 +229,7 @@ gb_file_selection_write_source (GtkWidget * widget,
   wname = g_strdup (data->wname);
 
   source_add (data, "\n");
-  child_name = gtk_widget_get_name (GTK_FILE_SELECTION (widget)->ok_button);
+  child_name = (gchar*) gtk_widget_get_name (GTK_FILE_SELECTION (widget)->ok_button);
   child_name = source_create_valid_identifier (child_name);
   source_add (data, "  %s = GTK_FILE_SELECTION (%s)->ok_button;\n",
 	      child_name, wname);
@@ -212,7 +237,7 @@ gb_file_selection_write_source (GtkWidget * widget,
   data->create_widget = FALSE;
   gb_widget_write_source (GTK_FILE_SELECTION (widget)->ok_button, data);
 
-  child_name = gtk_widget_get_name (GTK_FILE_SELECTION (widget)->cancel_button);
+  child_name = (gchar*) gtk_widget_get_name (GTK_FILE_SELECTION (widget)->cancel_button);
   child_name = source_create_valid_identifier (child_name);
   source_add (data, "  %s = GTK_FILE_SELECTION (%s)->cancel_button;\n",
 	      child_name, wname);
@@ -231,9 +256,9 @@ static GtkWidget *
 gb_file_selection_get_child (GtkWidget * widget,
 			     const gchar * child_name)
 {
-  if (!strcmp (child_name, FileSelOKButton))
+  if (!strcmp (child_name, GladeChildOKButton))
     return GTK_FILE_SELECTION (widget)->ok_button;
-  else if (!strcmp (child_name, FileSelCancelButton))
+  else if (!strcmp (child_name, GladeChildCancelButton))
     return GTK_FILE_SELECTION (widget)->cancel_button;
   else
     return NULL;
@@ -249,7 +274,8 @@ GbWidget *
 gb_file_selection_init ()
 {
   /* Initialise the GTK type */
-  gtk_file_selection_get_type ();
+  volatile GtkType type;
+  type = gtk_file_selection_get_type ();
 
   /* Initialize the GbWidget structure */
   gb_widget_init_struct (&gbwidget);
@@ -265,6 +291,7 @@ gb_file_selection_init ()
   gbwidget.gb_widget_set_properties = gb_file_selection_set_properties;
   gbwidget.gb_widget_get_child = gb_file_selection_get_child;
   gbwidget.gb_widget_write_source = gb_file_selection_write_source;
+  gbwidget.gb_widget_destroy = gb_window_destroy;
 /*
    gbwidget.gb_widget_create_popup_menu = gb_file_selection_create_popup_menu;
  */
