@@ -44,6 +44,13 @@ static gchar *Resizable = "FontSelDialog|GtkWindow::resizable";
 static gchar *DestroyWithParent = "FontSelDialog|GtkWindow::destroy_with_parent";
 static gchar *Icon = "FontSelDialog|GtkWindow::icon";
 
+static gchar *Role = "FontSelDialog|GtkWindow::role";
+static gchar *TypeHint = "FontSelDialog|GtkWindow::type_hint";
+static gchar *SkipTaskbar = "FontSelDialog|GtkWindow::skip_taskbar_hint";
+static gchar *SkipPager = "FontSelDialog|GtkWindow::skip_pager_hint";
+static gchar *Decorated = "FontSelDialog|GtkWindow::decorated";
+static gchar *Gravity = "FontSelDialog|GtkWindow::gravity";
+
 /******
  * NOTE: To use these functions you need to uncomment them AND add a pointer
  * to the funtion in the GbWidget struct at the end of this file.
@@ -86,6 +93,9 @@ gb_font_selection_dialog_new (GbWidgetNewData *data)
 			 data->action == GB_CREATING ? "font_selection" : NULL);
   gb_widget_set_child_name (fontsel->fontsel, GladeChildFontSelection);
 
+  gtk_object_set_data (GTK_OBJECT (new_widget), TypeHint,
+		       GINT_TO_POINTER (GLADE_TYPE_HINT_DIALOG_INDEX));
+
   return new_widget;
 }
 
@@ -103,7 +113,9 @@ gb_font_selection_dialog_create_properties (GtkWidget * widget,
 					DefaultWidth, DefaultHeight,
 					Shrink, Grow, AutoShrink,
 					WMName, WMClass,
-					Resizable, DestroyWithParent, Icon);
+					Resizable, DestroyWithParent, Icon,
+					Role, TypeHint, SkipTaskbar,
+					SkipPager, Decorated, Gravity);
 }
 
 
@@ -121,7 +133,9 @@ gb_font_selection_dialog_get_properties (GtkWidget *widget,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
 				     WMName, WMClass,
-				     Resizable, DestroyWithParent, Icon);
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 }
 
 
@@ -139,7 +153,9 @@ gb_font_selection_dialog_set_properties (GtkWidget * widget,
 				     DefaultWidth, DefaultHeight,
 				     Shrink, Grow, AutoShrink,
 				     WMName, WMClass,
-				     Resizable, DestroyWithParent, Icon);
+				     Resizable, DestroyWithParent, Icon,
+				     Role, TypeHint, SkipTaskbar,
+				     SkipPager, Decorated, Gravity);
 }
 
 
@@ -186,7 +202,9 @@ gb_font_selection_dialog_write_source (GtkWidget * widget,
 				   DefaultWidth, DefaultHeight,
 				   Shrink, Grow, AutoShrink,
 				   WMName, WMClass,
-				   Resizable, DestroyWithParent, Icon);
+				   Resizable, DestroyWithParent, Icon,
+				   Role, TypeHint, SkipTaskbar,
+				   SkipPager, Decorated, Gravity);
 
 
   /* We output the source code for the buttons here, but we don't want them
@@ -281,6 +299,7 @@ gb_font_selection_dialog_init ()
   gbwidget.gb_widget_set_properties	= gb_font_selection_dialog_set_properties;
   gbwidget.gb_widget_get_child		= gb_font_selection_dialog_get_child;
   gbwidget.gb_widget_write_source	= gb_font_selection_dialog_write_source;
+  gbwidget.gb_widget_destroy = gb_window_destroy;
 /*
   gbwidget.gb_widget_create_popup_menu	= gb_font_selection_dialog_create_popup_menu;
 */
