@@ -21,6 +21,7 @@
 #include <string.h>
 
 #include <gtk/gtkcombo.h>
+#include <gtk/gtkcombobox.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtklist.h>
@@ -131,9 +132,9 @@ gb_label_create_standard_properties (GtkWidget * widget,
 {
   GtkWidget *combo;
 
-  property_add_text (label_p, _("Label:"), _("The text to display"), 2);
+  property_add_text (label_p, _("Label:"), _("The text to display"), 5);
   property_add_bool (use_underline_p, _("Use Underline:"),
-		     _("If the text includes an underlined accelerator character"));
+		     _("If the text includes an underlined access key"));
   property_add_bool (use_markup_p, _("Use Markup:"),
 		     _("If the text includes pango markup"));
   property_add_choice (justify_p, _("Justify:"),
@@ -154,11 +155,11 @@ gb_label_create_standard_properties (GtkWidget * widget,
   property_add_int_range (ypad_p, _("Y Pad:"), _("The vertical padding"),
 			  0, 1000, 1, 10, 1);
   property_add_combo (focus_target_p, _("Focus Target:"),
-		      _("The widget to set the keyboard focus to when the underlined accelerator key is used"),
+		      _("The widget to set the keyboard focus to when the underlined access key is used"),
 		      NULL);
   combo = property_get_value_widget (focus_target_p);
-  gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (combo)->entry), FALSE);
-  gtk_combo_set_value_in_list (GTK_COMBO (combo), TRUE, TRUE);
+  gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (combo)->entry), FALSE);
+  /*gtk_combo_set_value_in_list (GTK_COMBO (combo), TRUE, TRUE);*/
 }
 
 
@@ -346,11 +347,17 @@ static void
 gb_label_get_focus_targets (GtkWidget * widget,
 			    GList ** focus_targets)
 {
-  if (GTK_WIDGET_CAN_FOCUS (widget) && GB_IS_GB_WIDGET (widget))
+  /* GtkWidget now has a "mnemonic_activate" signal, which some widgets use
+     to set focus to themselves or a child. So it is difficult to know if
+     a widget can be a focus target. We can't just use CAN_FOCUS as before.
+     For now I've just added some specific widgets. */
+  if (GB_IS_GB_WIDGET (widget)
+      && (GTK_WIDGET_CAN_FOCUS (widget)
+	  || GTK_IS_COMBO_BOX (widget)))
     {
       *focus_targets = g_list_insert_sorted (*focus_targets,
 					     (gchar*) gtk_widget_get_name (widget),
-					     g_str_equal);
+					     (GCompareFunc) g_utf8_collate);
     }
 
   if (GTK_IS_CONTAINER (widget))
@@ -582,7 +589,7 @@ gb_label_write_standard_source (GtkWidget * widget,
   gint i;
 
   if (gtk_object_get_data (GTK_OBJECT (widget), use_markup_p))
-    source_add (data, "  gtk_label_set_use_markup (%s, TRUE);\n",
+    source_add (data, "  gtk_label_set_use_markup (GTK_LABEL (%s), TRUE);\n",
 		data->wname);
 
   /* If there is an underlined accelerator, set up the accel signal.
@@ -611,7 +618,7 @@ gb_label_write_standard_source (GtkWidget * widget,
 	}
     }
 
-  if (GTK_LABEL (widget)->jtype != GTK_JUSTIFY_CENTER)
+  if (GTK_LABEL (widget)->jtype != GTK_JUSTIFY_LEFT)
     {
       for (i = 0; i < sizeof (GbJustifyValues) / sizeof (GbJustifyValues[0]);
 	   i++)
